@@ -48,21 +48,24 @@
 
 package com.openexchange.imageconverter.client;
 
+import static com.openexchange.imageconverter.client.ImageConverterClient.LOG;
 import static org.apache.commons.lang.StringUtils.indexOfIgnoreCase;
 import static org.apache.commons.lang.StringUtils.isNotEmpty;
 import static org.apache.commons.lang.StringUtils.lastIndexOfIgnoreCase;
+
 import java.io.BufferedReader;
 import java.io.StringReader;
 import java.net.URL;
 import java.util.concurrent.Callable;
 import java.util.concurrent.ExecutorService;
 import java.util.concurrent.Executors;
-import java.util.concurrent.FutureTask;
+import java.util.concurrent.Future;
 import java.util.concurrent.atomic.AtomicBoolean;
 import java.util.concurrent.atomic.AtomicInteger;
 import java.util.regex.Matcher;
 import java.util.regex.Pattern;
-import org.slf4j.Logger;
+
+import com.google.common.base.Throwables;
 import com.openexchange.annotation.NonNull;
 import com.openexchange.imageconverter.api.ImageServerUtil;
 import com.openexchange.imageconverter.client.generated.modules.ImageConverterApi;
@@ -76,19 +79,19 @@ import com.openexchange.imageconverter.client.generated.modules.ImageConverterAp
 class ImageConverterRemoteValidator {
 
     /**
-     * ESTABLISH_INITIAL_CONNECTION_DELAY_MILLIS
+     * ESTABLISH_CONNECTION_DELAY_MILLIS
      */
-    final protected static int ESTABLISH_INITIAL_CONNECTION_DELAY_MILLIS = 10000;
+    final protected static int ESTABLISH_CONNECTION_DELAY_MILLIS = 10000;
 
     /**
-     * ESTABLISH_INITIAL_CONNECTION_PERIOD_MILLIS
+     * ESTABLISH_CONNECTION_PERIOD_MILLIS
      */
-    final protected static int ESTABLISH_INITIAL_CONNECTION_PERIOD_MILLIS = 3000;
+    final protected static int ESTABLISH_CONNECTION_PERIOD_MILLIS = 3000;
 
     /**
-     * ESTABLISH_INITIAL_CONNECTION_TIMEOUT_MILLIS
+     * ESTABLISH_CONNECTION_TIMEOUT_MILLIS
      */
-    final protected static int ESTABLISH_INITIAL_CONNECTION_TIMEOUT_MILLIS = 10000;
+    final protected static int ESTABLISH_CONNECTION_TIMEOUT_MILLIS = 10000;
 
     /**
      * CHECK_REMOTE_CONNECTION_TIMEOUT_MILLIS
@@ -121,7 +124,6 @@ class ImageConverterRemoteValidator {
          */
         @Override
         public Integer call() throws Exception {
-            final Logger LOG = ImageConverterClient.getLogger();
             final ImageConverterApi imageConverterApi = m_imageConverterApiProvider.acquire();
             int remoteApiVersion = 0;
             String statusResponse = null;
@@ -143,7 +145,6 @@ class ImageConverterRemoteValidator {
                                 }
 
                                 if ((indexOf = lastIndexOfIgnoreCase(readLine, "api:")) > -1) {
-
                                     final Matcher matcher = API_VERSION_PATTERN.matcher(readLine.substring(indexOf));
 
                                     if (matcher.find()) {
@@ -157,17 +158,15 @@ class ImageConverterRemoteValidator {
                     LOG.trace("IC client RemoteValidator not able to get remote status of server", e);
                 } finally {
                     // disconnect and try again, if no valid answer has been received so far
-                    if (remoteApiVersion < 1) {
-                        if (m_remoteAPIVersion.get() > 0) {
-                            implSetConnectionLost();
+                    if (!implIsValidAPIVersion(remoteApiVersion)) {
+                        if (implIsValidAPIVersion(m_remoteAPIVersion.get())) {
+                            m_remoteAPIVersion.set(-1);
                         } else {
-                            synchronized (self) {
-                                final long curLogTime = System.currentTimeMillis();
+                            final long curLogTime = System.currentTimeMillis();
 
-                                if ((0 == m_lastLogTime) || ((curLogTime - m_lastLogTime) >= ENABLE_LOG_PERIOD_MILLIS)) {
-                                    LOG.error("IC client remote connection could not be established => please check remote ImageConverter server setup at: {}", m_remoteUrl);
-                                    m_lastLogTime = curLogTime;
-                                }
+                            if ((0 == m_lastLogTime) || ((curLogTime - m_lastLogTime) >= ENABLE_LOG_PERIOD_MILLIS)) {
+                                LOG.error("IC client remote connection could not be established => please check remote ImageConverter server setup at: {}", m_remoteUrl);
+                                m_lastLogTime = curLogTime;
                             }
                         }
                     }
@@ -176,15 +175,13 @@ class ImageConverterRemoteValidator {
                 }
             }
 
-            synchronized (self) {
-                if ((remoteApiVersion > 0) && (m_remoteAPIVersion.get() < 1)) {
-                    LOG.info("IC client established remote connection to ImageConverter server at: {}", m_remoteUrl);
-                }
-
-                m_remoteAPIVersion.set(remoteApiVersion);
-                m_running.compareAndSet(true, false);
+            if (implIsValidAPIVersion(remoteApiVersion) && !implIsValidAPIVersion(m_remoteAPIVersion.get())) {
+                LOG.info("IC client established remote connection to ImageConverter server at: {}", m_remoteUrl);
             }
 
+            m_remoteAPIVersion.set(remoteApiVersion);
+            m_validationRunning.compareAndSet(true, false);
+
             return Integer.valueOf(remoteApiVersion);
         }
     }
@@ -197,38 +194,18 @@ class ImageConverterRemoteValidator {
 
         m_imageConverterClientConfig = clientConfig;
         m_imageConverterApiProvider  = apiProvider;
+        m_callable = new ImageConverterServerValidatorCallable();
+        m_remoteUrl = m_imageConverterClientConfig.getImageConverterServerURL();
 
-        if (null != (m_remoteUrl = m_imageConverterClientConfig.getImageConverterServerURL())) {
-            m_callable = new ImageConverterServerValidatorCallable();
-
+        if (implIsValidRemoteUrl()) {
             // initial trigger at end of construction
-            implEstablishInitialConnection(ESTABLISH_INITIAL_CONNECTION_TIMEOUT_MILLIS);
-        } else {
-            m_callable = null;
+            m_executorService.submit(() -> {
+                Thread.sleep(Math.max(0, ESTABLISH_CONNECTION_DELAY_MILLIS));
+                return Boolean.valueOf(getRemoteAPIVersion() >= 1);
+            });
         }
     }
 
-    /**
-     * @return
-     */
-    ImageConverterClientConfig getConfig() {
-        return m_imageConverterClientConfig;
-    }
-
-    /**
-     * @return
-     */
-    URL getRemoteUrl() {
-        return m_remoteUrl;
-    }
-
-    /**
-     * @return
-     */
-    boolean isRemoteServerSupported() {
-        return (null != m_callable);
-    }
-
     /**
      * @return
      */
@@ -240,59 +217,76 @@ class ImageConverterRemoteValidator {
      * @return
      */
     synchronized int getRemoteAPIVersion() {
-        if (isRemoteServerSupported() && (m_remoteAPIVersion.get() < 1) && m_running.compareAndSet(false, true)) {
-            m_executorService.submit(new FutureTask<>(m_callable));
+        int ret = m_remoteAPIVersion.get();
+
+        if (implIsValidRemoteUrl() && !implIsValidAPIVersion(ret)) {
+            final Future<Integer> resultFuture = m_executorService.submit(m_callable);
+
+            if (null != resultFuture) {
+                try {
+                    ret = resultFuture.get().intValue();
+                } catch (Exception e) {
+                    LOG.error("IC client remote validator caught exception: {}", Throwables.getRootCause(e));
+                }
+            }
         }
 
-        return m_remoteAPIVersion.get();
+        return ret;
+    }
+
+    /**
+     * updateRemoteURL
+     *
+     * @param remoteUrl
+     */
+    synchronized void updateRemoteURL(final URL remoteUrl) {
+        if ((null != remoteUrl) && (!implIsValidRemoteUrl() || !m_remoteUrl.equals(remoteUrl))) {
+
+            m_remoteAPIVersion.set(-1);
+            m_validationRunning.set(false);
+            m_remoteUrl = (null != remoteUrl) && (remoteUrl.toString().length() > 0) ? remoteUrl : null;
+
+            // trigger new connection validation and get current API version
+            final int newConnectionAPIVersion = getRemoteAPIVersion();
+
+            if (implIsValidAPIVersion(newConnectionAPIVersion)) {
+                LOG.info("IC client successfully connected to remote server after configuration update.");
+            } else {
+                LOG.error("IC client could not connect to remote server after configuration update!");
+            }
+        }
     }
 
     /**
      *
      */
-    synchronized void resetConnection() {
-        if ((null != m_callable) && (m_remoteAPIVersion.get() >= 1) && !m_running.get()) {
-            implSetConnectionLost();
+    synchronized void connectionLost() {
+        if (implIsValidRemoteUrl() && implIsValidAPIVersion(m_remoteAPIVersion.get())) {
+            m_remoteAPIVersion.set(-1);
         }
     }
 
     // - Implementation --------------------------------------------------------
 
     /**
-     * @param timeoutMillis
+     * implIsValidRemoteUrl
+     *
+     * @return
      */
-    protected void implEstablishInitialConnection(final long timeoutMillis) {
-
-        m_executorService.submit(new Callable<Boolean>() {
-
-            @Override
-            public Boolean call() throws Exception {
-                Thread.sleep(Math.max(0, ESTABLISH_INITIAL_CONNECTION_DELAY_MILLIS - ESTABLISH_INITIAL_CONNECTION_PERIOD_MILLIS));
-
-                final long endTryTimeMillis = System.currentTimeMillis() + timeoutMillis;
-                boolean ret = false;
-
-                do {
-                    Thread.sleep(ESTABLISH_INITIAL_CONNECTION_PERIOD_MILLIS);
-                    ret = (getRemoteAPIVersion() >= 1);
-                } while (!ret && (System.currentTimeMillis() < endTryTimeMillis));
-
-                return Boolean.valueOf(ret);
-            }
-        });
+    private boolean implIsValidRemoteUrl() {
+        return (null != m_remoteUrl);
     }
 
     /**
+     * implIsValidAPIVersion
      *
+     * @param apiVersion
+     * @return
      */
-    protected void implSetConnectionLost() {
-        m_remoteAPIVersion.set(-1);
-
-        ImageConverterClient.getLogger().error("IC client lost remote connection to remote URL: {}", m_remoteUrl);
+    private static boolean implIsValidAPIVersion(final int apiVersion) {
+        return (apiVersion > 0);
     }
 
-    // - Static members --------------------------------------------------------
-
     final protected static Pattern API_VERSION_PATTERN = Pattern.compile("[aA][pP][iI]\\:\\s*v([0-9]+)");
 
     // - Members ---------------------------------------------------------------
@@ -301,7 +295,7 @@ class ImageConverterRemoteValidator {
 
     final protected ExecutorService m_executorService = Executors.newCachedThreadPool();
 
-    final protected AtomicBoolean m_running = new AtomicBoolean(false);
+    final protected AtomicBoolean m_validationRunning = new AtomicBoolean(false);
 
     final protected AtomicInteger m_remoteAPIVersion = new AtomicInteger(-1);
 
@@ -309,9 +303,9 @@ class ImageConverterRemoteValidator {
 
     final protected ImageConverterApiProvider m_imageConverterApiProvider;
 
-    final protected URL m_remoteUrl;
-
     final protected ImageConverterServerValidatorCallable m_callable;
 
+    protected URL m_remoteUrl;
+
     protected long m_lastLogTime = 0;
 }
